@@ -1,8 +1,9 @@
 import ctypes
-from pkg_resources import resource_filename
-import PySimpleGUI as sg
+import json
 import os
-import ujson
+
+import PySimpleGUI as sg
+from pkg_resources import resource_filename
 
 
 def data_path(*args):
@@ -24,7 +25,7 @@ cfg = {
 
     # Vocabularies
     'vocab-file-types': [['Vocabulary Files', '*.cson']],
-    
+
     # Sets
     'common-sets': [data_path('Default Assets')],
     'gif-pause-duration': 1.2,
@@ -36,8 +37,8 @@ cfg = {
     'critical-word-weight': 3.5,
 
     # Locale
-    'locale': ['Pup Quiz', 'Click on a vocabulary to quiz for it.\nAdd a vocabulary by selecting an empty slot.\nDelete a vocabulary with a right-click.', '{} - Pup Quiz', 'Welcome to Pup Quiz!', 'Correct! {:.2f}%', '{}', 'Added to word stock', 'Got it!', 'Submit', 'Translate', 'Translation opened', 'Reset', 'Menu', 'Are you sure you want to delete "{}"?', 'Are you sure you want to reset progress on "{}"?'],
-    
+    'locale': ['Pup Quiz', 'Click on a vocabulary to quiz for it.\nAdd a vocabulary by selecting an empty slot.\nDelete a vocabulary with a right-click.', '{} - Pup Quiz', 'Welcome to Pup Quiz!', 'Correct!', '{}', 'Added to word stock', 'Got it!', 'Submit', 'Translate', 'Translation opened', 'Reset', 'Menu', 'Are you sure you want to delete "{}"?', 'Are you sure you want to reset progress on "{}"?'],
+
     # Styling
     'app-icon': resource_filename('pupquiz.resources', 'icon.ico'),
     'image-folder-on': resource_filename('pupquiz.resources', 'folder-on.png'),
@@ -76,6 +77,9 @@ cfg = {
     'thumbnail-undone-brightness': 0.75,
     'thumbnail-undone-saturation': 0.4,
     'thumbnail-undone-blur-radius': 3,
+    'background-brightness': 0.6,
+    'background-saturation': 0.8,
+    'background-blur-radius': 10,
 }
 
 
@@ -94,10 +98,10 @@ if not os.path.isdir(data_path('Default Assets')):
 
 if not os.path.isfile(data_path('config-default.json')):
     with open(data_path('config-default.json'), 'w') as f:
-        ujson.dump(cfg, f, indent=4)  # for reference
+        json.dump(cfg, f, indent=4)  # for reference
 try:
     with open(CFG_PATH, 'r') as f:
-        cfg.update(ujson.load(f) or {})  # apply user preference
+        cfg.update(json.load(f) or {})  # apply user preference
 except FileNotFoundError:
     with open(CFG_PATH, 'w') as f:
         f.write('{\n}')
