@@ -1,7 +1,8 @@
 import ctypes
-from .config import CFG_PATH
 import glob
+import json
 import os
+import pickle
 import re
 from itertools import chain, groupby, islice
 from operator import itemgetter
@@ -10,8 +11,6 @@ from typing import Dict, Iterator, List, Optional, Tuple
 
 import PySimpleGUI as sg
 
-import ujson
-
 from .config import (CFG_APPNAME, CFG_CONFIRM_DELETE, CFG_CONFIRM_RESET,
                      CFG_PATH, CFG_SELECT_INFO, cfg, data_path)
 from .graphics import encode, get_def_thumbnail, make_thumbnail
@@ -46,13 +45,48 @@ vconfigsearch = re.compile(cfg['patt-vocab-config']).search
 Set = List[str]
 
 
+class SetProvider:
+    def __init__(self, v):
+        self.__v = v
+        self.__sets = {
+            **get_sets(os.path.dirname(v[VOCAB_PATH])), **common_sets}
+        self.__keys = Random(ses[SES_VOCABS].index(v)).sample(
+            [*self.__sets], len(self.__sets))
+
+        # Remove references to outdated sets (gallery info comes from disk)
+        self.__gallery = v[VOCAB_GALLERY] = [
+            *({*v[VOCAB_GALLERY]} & {*self.__keys})]
+
+        # Order keys so that unlocked are first, then locked
+        self.__keys = sorted(
+            self.__keys, key=lambda x: x not in self.__gallery)
+
+    def reset_progress(self) -> bool:
+        'Clears gallery, moves words to the first bucket. Calls update_vocab.'
+        if sg.popup_ok_cancel(CFG_CONFIRM_RESET.format(self.__v[VOCAB_NAME]), title=CFG_APPNAME) != 'OK':
+            return False
+        self.__keys.extend(self.__gallery)
+        self.__gallery.clear()
+        for l in self.__v[VOCAB_WORDS]:
+            l *= 0
+        update_vocab(self.__v)
+        return True
+
+    def get_image(self, step: float) -> Tuple[List[str], int]:
+        'Returns set and image index associated with given index value between [0,1).'
+        s, i = divmod(len(self.__sets)*step, 1)
+        key = self.__keys[int(s)]
+        s = self.__sets[key]
+        return [f'{key}{v}' for v in s], int(i*len(s))
+
+
 def new_vocab() -> dict:
     return {VOCAB_WORDS: [[], [], [], [], [], []], VOCAB_GALLERY: []}
 
 
 try:
-    with open(data_path('session.json'), 'r') as f:
-        ses = ujson.load(f)
+    with open(data_path('session'), 'rb') as f:
+        ses = pickle.load(f)
 except FileNotFoundError:
     ses = {SES_LAST_DIR: '', SES_CFG_DATE: 0, SES_SELECT_GUIDE_HEIGHT: None, SES_WIN_POS: (None, None),
            SES_VOCABS: [new_vocab() for _ in range(NVOCABS)]}
@@ -71,19 +105,21 @@ def get_sets(*paths) -> Dict[str, List[str]]:
 
 
 def calc_progress(words):
-    return sum(i*len(l) for i, l in enumerate(words)) / (sum(map(len, words))*(len(words)-1))
+    nwords = sum(map(len, words))
+    nsteps = nwords * (len(words)-2)
+    return sum(i*len(l) for i, l in enumerate(words[2:], start=1)) / nsteps
 
 
 def save_session_json():
-    with open(data_path('session.json'), 'w') as f:
-        ujson.dump(ses, f)
+    with open(data_path('session'), 'wb') as f:
+        pickle.dump(ses, f)
 
 
 def update_thumbnail(v: dict):
-    img = next(islice(common_sets.items(), ses[SES_VOCABS].index(v), None))
-
+    progress = calc_progress(v[VOCAB_WORDS])
+    set_, img = SetProvider(v).get_image(progress)
     v[VOCAB_ICON_ON], v[VOCAB_ICON_OFF] = make_thumbnail(
-        os.path.join(img[0], img[1][0]), v[VOCAB_NAME], calc_progress(v[VOCAB_WORDS]))
+        set_[img], v[VOCAB_NAME], progress)
 
 
 def update_vocab(v: dict):
@@ -108,7 +144,7 @@ def update_vocab(v: dict):
     v[VOCAB_WORDS][0] = [*words]
     v[VOCAB_DATE] = os.path.getmtime(v[VOCAB_PATH])
     v[VOCAB_NAME] = title
-    v[VOCAB_CONFIG] = ujson.loads(mconfig[1]) if mconfig else {}
+    v[VOCAB_CONFIG] = json.loads(mconfig[1]) if mconfig else {}
     update_thumbnail(v)
 
 
@@ -137,51 +173,6 @@ if ses[SES_CFG_DATE] < os.path.getmtime(CFG_PATH):
 def_on, def_off = ses[SES_ADD_V_HOVER], ses[SES_ADD_V_DEF]
 
 
-class SetProvider:
-    def __init__(self, v):
-        self.__v = v
-        self.__sets = {
-            **get_sets(os.path.dirname(v[VOCAB_PATH])), **common_sets}
-        self.__rd = Random(ses[SES_VOCABS].index(v))
-        self.keys = self.__rd.sample([*self.__sets], len(self.__sets))
-
-        # Remove references to outdated sets (gallery info comes from disk)
-        self.__gallery = v[VOCAB_GALLERY] = [
-            *({*v[VOCAB_GALLERY]} & {*self.keys})]
-
-        # Order keys so that unlocked are first, then locked
-        self.keys = sorted(self.keys, key=lambda x: x not in self.__gallery)
-
-    @property
-    def rd(self):
-        'A random device seeded by vocabulary index'
-        return self.__rd
-
-    def unlocked(self, item: Optional[str] = None):
-        if item is None:
-            return self.__gallery
-        self.__gallery.append(item)
-
-    def reset_progress(self) -> bool:
-        'Clears gallery, moves words to the first bucket. Calls update_vocab.'
-        if sg.popup_ok_cancel(CFG_CONFIRM_RESET.format(self.__v[VOCAB_NAME]), title=CFG_APPNAME) != 'OK':
-            return False
-        self.keys.extend(self.__gallery)
-        self.__gallery.clear()
-        for l in self.__v[VOCAB_WORDS]:
-            l *= 0
-        update_vocab(self.__v)
-        return True
-
-    def __getitem__(self, key_: int) -> Set:
-        key = self.keys[key_]
-        return [f'{key}{v}' for v in self.__sets[key]]
-
-    def __len__(self) -> int:
-        'Returns the amount of sets in total (locked+unlocked)'
-        return self.__sets.__len__()
-
-
 def get_vocabulary(event: Optional[int] = None) -> Tuple[dict, SetProvider]:
     'Presents vocabulary selection screen and returns chosen vocab + its sets'
 
@@ -223,7 +214,7 @@ def get_vocabulary(event: Optional[int] = None) -> Tuple[dict, SetProvider]:
         while True:
             event, values = win.read(timeout=500)
             if event is None:
-                win.close()
+                win.TKroot.destroy()
                 del win
                 return None, None
             ses[SES_WIN_POS] = list(win.CurrentLocation())
@@ -242,6 +233,7 @@ def get_vocabulary(event: Optional[int] = None) -> Tuple[dict, SetProvider]:
                 if all(not x for x in v[VOCAB_WORDS][:-1]):
                     remove_vocab(event, win)
                     continue
+                win.TKroot.destroy()
                 win.close()
                 del win
                 return v, SetProvider(v)
